#clear workspace
rm(list = ls())
#set true to export png
DoPNG <- TRUE 
#DoPNG <- FALSE

#needed packages
if (!require("plyr")) { 
   install.packages("plyr", dependencies = TRUE)
   library(plyr)
   } #used for revalue
if (!require("plotrix")) { 
   install.packages("plotrix", dependencies = TRUE)
   library(plotrix)
   } #used for plotCI

#helper function
se<-function(x) { #standard error 
		  s <- sd(x,na.rm=TRUE)
		  l <- length(x[!is.na(x)])
		  s/sqrt(l)
}
ci<-function(x) { #confidence interval of t-distriubution at 5%
		  s <- sd(x,na.rm=TRUE)
		  l <- length(x[!is.na(x)])
		  qt(0.975,df=l-1)*s/sqrt(l) #95% confidence interval
}
signify<-function(ax,ay,at) {
  w=0.02
  h=0.1
  lines(x=c(ax,ax),y=c(ay-h,ay+h))
  lines(x=c(ax-w,ax),y=c(ay-h,ay-h))
  lines(x=c(ax-w,ax),y=c(ay+h,ay+h))
  text(ax+0.05,ay,at)
}

#load data
print("Reading LibO data...")
libodata <- read.table("results-survey783768.csv",header=TRUE,sep=",")
colnames(libodata) <- c("ID", "Date", "Lastpage", "Language","UX","Replicability","Transparency",
                                                 "TrustedProducer","TechSupport","Stability","Security","Cost","Compatibility",
                                                 "Customizability", "Apache OpenOffice","Apple iWork","Calligra","Corel WordPerfect",
                                                 "Google Docs","Kingsoft WPS","Microsoft Office","NeoOffice",
                                                 "OnlyOffice","SoftMaker","Other","Online","Continuance","Expert","Version",
                                                 "Normal User","Administration.","Development/Design","HelpAuthoring",
                                                 "Marketing","Localization","QA","Training","Member","OS","OtherOS")
#ignore no input  
libodata <- subset(libodata, Lastpage == 1)
#mapping No & N/A = 0 Yes = 1 for easy sum
a <- which(colnames(libodata)=="Apache OpenOffice")
o <- which(colnames(libodata)=="SoftMaker")
for (i in a:o) libodata[i] <- as.integer(revalue(libodata[,i],c("No"=0,"N/A"=0,"Yes"=1)))-1 
a <- which(colnames(libodata)=="Normal User")
o <- which(colnames(libodata)=="Training")
for (i in a:o) libodata[i] <- as.integer(revalue(libodata[,i],c("No"=0,"N/A"=0,"Yes"=1)))-1 

#recode and invert to 1..5 
codes <- c("Extremely important"="1","Very important"="2","Moderately important"="3",
                      "Slightly important"="4","Not at all important"="5")
lev <- c("Not at all important","Slightly important","Moderately important","Very important","Extremely important")
a <- which(colnames(libodata)=="UX")
o <- which(colnames(libodata)=="Customizability")
for (i in a:o) {
  libodata[,i] <- factor(libodata[,i], levels=lev)
  libodata[,i] <- as.numeric(revalue(libodata[,i], codes))
}

#load data
print("Reading Github data")
#get data from http://opensourcesurvey.org/2017/#data
if (!file.exists("survey_data.csv")) print("Get the Github raw data from http://opensourcesurvey.org/2017/#data")
github <- read.table("survey_data.csv",header=TRUE,sep=",",fill=TRUE,quote='"')
a<-which(colnames(github)=="USER.VALUES.STABILITY")
o<-which(colnames(github)=="USER.VALUES.TRUSTED.PRODUCER")
for (i in a:o) {
  github[,i] <- factor(github[,i], levels=lev)
  github[,i] <- as.numeric(revalue(github[,i], codes))
}
github <- github[a:o]
#consistent col names
colnames(github) <- c("Stability","Innovation","Replicability","Compatibility","Security","Cost","Transparency","UX",
                                              "Customizability","TechSupport","TrustedProducer")
github <- github[,order(colMeans(github,na.rm=TRUE))]

if (DoPNG) png("Importance.png",width=600,height=600)
par(mar=c(5.1, 8.1, 4.1, 2.1))
#base plot of github data in gray
plotCI(y=1:11,x=sapply(github,mean,na.rm=TRUE),uiw=sapply(github,ci),err="x",
              xlim=c(3.0,4.75),col="grey",xlab="",ylab="",xaxt="n",yaxt="n")
#libo data are fitted into this plot
for(i in 11:1) {
  z <- NA
  z <- which( colnames(libodata)==colnames(github)[i])
  if (length(z)>0) plotCI(y=i-0.1,x=sapply(libodata[z],mean,na.rm=TRUE),uiw=sapply(libodata[z],ci),err="x",
                                                xlim=c(3.0,4.75),col="black",add=TRUE,lwd=2)
}
title(main="Importance of aspects")
axis(side=1, at=c(3,4.5),labels=c("less important","more important"), cex=0.7, las=1)
axis(side=2, at=1:11, labels=colnames(github), cex=0.7, las=2)
legend("bottomright",c("Github","LibreOffice"),lwd=c(1,2),col=c("grey","black"),pch=1)
if (DoPNG) dev.off()

# ------ Other office suites ------ #
a <- which(colnames(libodata)=="Apache OpenOffice")
o <- which(colnames(libodata)=="SoftMaker")
z <- NA
for (i in a:o) z[(i-a)+1] <- sum(libodata[i])
z[(o-a)+2]<-sum(libodata$Other != "") #Other
#plot
if (DoPNG) png("OfficeSuites.png",width=600,height=600)
plot.new()
floating.pie(0.24,0.7,z[order(z)],radius=0.25) #plotrix
#number of multiple answers
n <- libodata[a]
for (i in (a+1):o) n <- n + libodata[i]
n <- table(n)
floating.pie(0.76,0.7,n,radius=0.25)
legend(x=0.0,y=0.41,paste(colnames(libodata[a:(o+1)])[order(z)],': ',z[order(z)],sep=""),fill=rainbow(length(z)),bty="n")
legend(x=0.65,y=0.42,paste("n=",names(n),": ",n,sep=""),fill=rainbow(length(z)),bty="n")
title(main="Usage of other office suites")
if (DoPNG) dev.off()

# ------ Affiliation ------ #
a <- which(colnames(libodata)=="Normal User")
o <- which(colnames(libodata)=="Training")
z <- NA
for (i in a:o) z[(i-a)+1] <- sum(libodata[i])
#plot
if (DoPNG) png("Affiliation.png",width=600,height=600)
plot.new()
floating.pie(0.24,0.7,z[order(z)],radius=0.25) #plotrix
#number of multiple answers
n <- libodata[a]
for (i in (a+1):o) n <- n + libodata[i]
n <- table(n)
floating.pie(0.76,0.7,n,radius=0.25)
legend(x=0.0,y=0.41,paste(colnames(libodata[a:o])[order(z)],': ',z[order(z)],sep=""),fill=rainbow(length(z)),bty="n")
legend(x=0.65,y=0.42,paste("n=",names(n),": ",n,sep=""),fill=rainbow(length(z)),bty="n")
title(main="Affiliation with the project")
if (DoPNG) dev.off()

# ------ Continuance ------ #
if (DoPNG) png("Continuance.png",width=600,height=600)
par(mar=c(5.1, 4.1, 4.1, 2.1))
barplot(table(libodata$Continuance[libodata$Continuance<1000]))
title(main="Usage of LibreOffice and predecessors",xlab="Years",ylab="[n]")
if (DoPNG) dev.off()

# ------ OS ------ 
codes <- c("Microsoft Windows 7"="Windows","Microsoft Windows 8, 10"="Windows","Microsoft Windows XP, 2000, Vista, NT"="Windows","Linux"="Linux","Other"=NA,"macOS"=NA)#"macOS"="macOS"
libodata$OS_WML <- revalue(libodata$OS, codes)

man <- manova(cbind(UX, Replicability, Transparency,  TrustedProducer, TechSupport, Stability, 
                                              Security, Cost, Compatibility) ~ OS_WML, data=libodata)
summary(man)
summary.aov(man)

if (DoPNG) png("OS.png",width=600,height=600)
par(mar=c(5.1, 8.1, 4.1, 2.1))
cols <- c(9,8,6,12,14,7,13,5,10,11) #sorted according first plot
z <- subset(libodata,OS_WML=="Windows")[cols]
plotCI(y=1.1:10.1,
              x=sapply(z,mean,na.rm=TRUE),
              uiw=sapply(z,ci),
              err="x",col="blue",xlab="",ylab="",xaxt="n",yaxt="n",
              xlim=c(2.75,4.75),ylim=c(0.5,10.5))
z <- subset(libodata,OS_WML=="Linux")[cols]
plotCI(y=0.9:9.9,
              x=sapply(z,mean,na.rm=TRUE),
              uiw=sapply(z,ci),
              err="x",col="red",add=TRUE)
title(main="Importance in relation to operating system")
axis(side=1, at=c(2.8,4.5),labels=c("less important","more important"), cex=0.7, las=1)
axis(side=2, at=1:10, labels=colnames(libodata)[cols], cex=0.7, las=2)
legend("bottomright",c("Windows","Linux"),lwd=c(1,2),col=c("blue","red"),pch=1)
#significance indicators
signify(4.1,6,"***") #Transparency
signify(3.8,4,"***") #Costs
signify(3.75,3,"***") #Replicability
par(mar=c(5.1, 4.1, 4.1, 2.1))
if (DoPNG) dev.off()

# ----- Member -----
man <- manova(cbind(UX, Replicability, Transparency,  TrustedProducer, TechSupport, Stability, 
                                              Security, Cost, Compatibility) ~ Member, data=libodata)
summary(man)
summary.aov(man)

if (DoPNG) png("Member.png",width=600,height=600)
par(mar=c(5.1, 8.1, 4.1, 2.1))
cols <- c(9,8,6,12,14,7,13,5,10,11) #sorted according first plot
z <- subset(libodata,Member=="Yes")[cols]
plotCI(y=1.1:10.1,
              x=sapply(z,mean,na.rm=TRUE),
              uiw=sapply(z,ci),
              err="x",col="blue",xlab="",ylab="",xaxt="n",yaxt="n",
              xlim=c(2.75,4.75),ylim=c(0.5,10.5))
z <- subset(libodata,Member=="No")[cols]
plotCI(y=0.9:9.9,
              x=sapply(z,mean,na.rm=TRUE),
              uiw=sapply(z,ci),
              err="x",col="red",add=TRUE)
title(main="Importance in relation to membership")
axis(side=1, at=c(2.8,4.5),labels=c("less important","more important"), cex=0.7, las=1)
axis(side=2, at=1:10, labels=colnames(libodata)[cols], cex=0.7, las=2)
legend("bottomright",c("Yes","No"),lwd=c(1,2),col=c("blue","red"),pch=1)
#significance indicators
signify(3.8,1,"**") #Tech support
signify(4.5,8,"**") #UX
#signify(3.75,3,"***") #Replicability
par(mar=c(5.1, 4.1, 4.1, 2.1))
if (DoPNG) dev.off()

# ----- Version -----
codes <- c("Bleeding edge with nightly builds"=NA,"I don't care at all"=NA,
                    "I'm always on 'stable'"="stable","No idea, whatever the updater recommends"=NA,
                    "Nothing else than 'fresh'"="fresh")
libodata$Vers <- revalue(libodata$Version, codes)
man <- manova(cbind(UX, Replicability, Transparency,  TrustedProducer, TechSupport, Stability, 
                                              Security, Cost, Compatibility) ~ Vers, data=libodata)
summary(man)
summary.aov(man)

if (DoPNG) png("Version.png",width=600,height=600)
par(mar=c(5.1, 8.1, 4.1, 2.1))
cols <- c(9,8,6,12,14,7,13,5,10,11) #sorted according first plot
z <- subset(libodata,Vers=="stable")[cols]
plotCI(y=1.1:10.1,
              x=sapply(z,mean,na.rm=TRUE),
              uiw=sapply(z,ci),
              err="x",col="blue",xlab="",ylab="",xaxt="n",yaxt="n",
              xlim=c(2.75,4.75),ylim=c(0.5,10.5))
z <- subset(libodata,Vers=="fresh")[cols]
plotCI(y=0.9:9.9,
              x=sapply(z,mean,na.rm=TRUE),
              uiw=sapply(z,ci),
              err="x",col="red",add=TRUE)
title(main="Importance in relation to version")
axis(side=1, at=c(2.8,4.5),labels=c("less important","more important"), cex=0.7, las=1)
axis(side=2, at=1:10, labels=colnames(libodata)[cols], cex=0.7, las=2)
legend("bottomright",c("stable","fresh"),lwd=c(1,2),col=c("blue","red"),pch=1)
#significance indicators
signify(4.75,9,"**") #Stability
signify(4.6,8,"*") #UX
par(mar=c(5.1, 4.1, 4.1, 2.1))
if (DoPNG) dev.off()

# ----- Expertise -----
man <- manova(cbind(UX, Replicability, Transparency,  TrustedProducer, TechSupport, Stability, 
                                              Security, Cost, Compatibility) ~ Expert, data=libodata)
summary(man)
summary.aov(man)

if (DoPNG) png("Expertise.png",width=600,height=600)
par(mar=c(5.1, 8.1, 4.1, 2.1))
cols <- c(9,8,6,12,14,7,13,5,10,11) #sorted according first plot
z <- subset(libodata,Expert=="No")[cols]
plotCI(y=1.1:10.1,
              x=sapply(z,mean,na.rm=TRUE),
              uiw=sapply(z,ci),
              err="x",col="blue",xlab="",ylab="",xaxt="n",yaxt="n",
              xlim=c(2.75,4.75),ylim=c(0.5,10.5))
z <- subset(libodata,Expert=="Yes")[cols]
plotCI(y=0.9:9.9,
              x=sapply(z,mean,na.rm=TRUE),
              uiw=sapply(z,ci),
              err="x",col="red",add=TRUE)
title(main="Importance in relation to expertise")
axis(side=1, at=c(2.8,4.5),labels=c("less important","more important"), cex=0.7, las=1)
axis(side=2, at=1:10, labels=colnames(libodata)[cols], cex=0.7, las=2)
legend("bottomright",c("no","yes"),lwd=c(1,2),col=c("blue","red"),pch=1)
#significance indicators
signify(4.1,2,"*") #TrustedProducer
signify(3.55,3,"**") #Replicability
signify(3.95,6,"***") #Transparency
par(mar=c(5.1, 4.1, 4.1, 2.1))
if (DoPNG) dev.off()
